@@ -1,4 +1,5 @@
-use std::{net::{SocketAddr}, sync::{Arc}};
+use std::{net::{SocketAddr}, sync::{Arc, atomic::{AtomicU8, AtomicI64}}};
+use rand::Rng;
 use tokio::{net::UdpSocket, sync::{Mutex, mpsc::channel}, time::{sleep, timeout}};
 
 use tokio::sync::mpsc::{Sender, Receiver};
@@ -15,9 +16,13 @@ pub struct RaknetSocket{
     recvq : Arc<Mutex<RecvQ>>,
     sendq : Arc<Mutex<SendQ>>,
     connected : Arc<AtomicBool>,
+    last_heartbeat_time : Arc<AtomicI64>,
+    enable_loss : Arc<AtomicBool>,
+    loss_rate : Arc<AtomicU8>
 }
 
 impl RaknetSocket {
+    
     pub fn from(addr : &SocketAddr , s : &Arc<UdpSocket> ,receiver : Receiver<Vec<u8>> , mtu : u16 , collecter : Arc<Mutex<Sender<SocketAddr>>>) -> Self {
 
         let (user_data_sender , user_data_receiver) =  channel::<Vec<u8>>(100);
@@ -30,6 +35,9 @@ impl RaknetSocket {
             recvq : Arc::new(Mutex::new(RecvQ::new())),
             sendq : Arc::new(Mutex::new(SendQ::new(mtu))),
             connected : Arc::new(AtomicBool::new(true)),
+            last_heartbeat_time : Arc::new(AtomicI64::new(cur_timestamp_millis())),
+            enable_loss : Arc::new(AtomicBool::new(false)),
+            loss_rate : Arc::new(AtomicU8::new(0))
         };
         ret.start_receiver(receiver , user_data_sender);
         ret.start_tick(Some(collecter));
@@ -39,7 +47,7 @@ impl RaknetSocket {
     async fn handle (frame : &FrameSetPacket , peer_addr : &SocketAddr , local_addr : &SocketAddr , sendq : &Mutex<SendQ> , user_data_sender : &Sender<Vec<u8>>) -> Result<bool> {
         match PacketID::from(frame.data[0])? {
             PacketID::ConnectionRequest => {
-                let packet = read_packet_connection_request(frame.data.as_slice()).await.unwrap();
+                let packet = read_packet_connection_request(frame.data.as_slice()).await?;
                 
                 let packet_reply = ConnectionRequestAccepted{
                     client_address: *peer_addr,
@@ -48,11 +56,11 @@ impl RaknetSocket {
                     accepted_timestamp: cur_timestamp_millis(),
                 };
 
-                let buf = write_packet_connection_request_accepted(&packet_reply).await.unwrap();
-                sendq.lock().await.insert(Reliability::ReliableOrdered,&buf, cur_timestamp_millis());
+                let buf = write_packet_connection_request_accepted(&packet_reply).await?;
+                sendq.lock().await.insert(Reliability::ReliableOrdered,&buf)?;
             },
             PacketID::ConnectionRequestAccepted => {
-                let packet = read_packet_connection_request_accepted(frame.data.as_slice()).await.unwrap();
+                let packet = read_packet_connection_request_accepted(frame.data.as_slice()).await?;
                 
                 let packet_reply = NewIncomingConnection{
                     server_address: *local_addr,
@@ -62,30 +70,30 @@ impl RaknetSocket {
 
                 let mut sendq = sendq.lock().await;
 
-                let buf = write_packet_new_incomming_connection(&packet_reply).await.unwrap();
-                sendq.insert(Reliability::ReliableOrdered ,&buf, cur_timestamp_millis());
+                let buf = write_packet_new_incomming_connection(&packet_reply).await?;
+                sendq.insert(Reliability::ReliableOrdered ,&buf)?;
 
                 let ping = ConnectedPing{
                     client_timestamp: cur_timestamp_millis(),
                 };
 
                 //i dont know why incomming packet after always follow a connected ping packet in minecraft bedrock 1.18.12.
-                let buf = write_packet_connected_ping(&ping).await.unwrap();
-                sendq.insert(Reliability::Unreliable ,&buf, cur_timestamp_millis());
+                let buf = write_packet_connected_ping(&ping).await?;
+                sendq.insert(Reliability::Unreliable ,&buf)?;
             }
             PacketID::NewIncomingConnection => {
-                let _packet = read_packet_new_incomming_connection(frame.data.as_slice()).await.unwrap();
+                let _packet = read_packet_new_incomming_connection(frame.data.as_slice()).await?;
             }
             PacketID::ConnectedPing => {
-                let packet = read_packet_connected_ping(frame.data.as_slice()).await.unwrap();
+                let packet = read_packet_connected_ping(frame.data.as_slice()).await?;
                 
                 let packet_reply = ConnectedPong{
                     client_timestamp: packet.client_timestamp,
                     server_timestamp: cur_timestamp_millis(),
                 };
 
-                let buf = write_packet_connected_pong(&packet_reply).await.unwrap();
-                sendq.lock().await.insert(Reliability::Unreliable ,&buf, cur_timestamp_millis());
+                let buf = write_packet_connected_pong(&packet_reply).await?;
+                sendq.lock().await.insert(Reliability::Unreliable ,&buf)?;
             }
             PacketID::ConnectedPong => {}
             PacketID::Disconnect => {
@@ -102,7 +110,25 @@ impl RaknetSocket {
         }
         Ok(true)
     }
+
+    pub fn set_loss_rate(&mut self ,stage : u8){
+        self.enable_loss.store(true, Ordering::Relaxed);
+        self.loss_rate.store(stage, Ordering::Relaxed);
+
+    }
     
+    async fn sendto(s : &UdpSocket , buf : &[u8] , target : &SocketAddr , enable_loss : &AtomicBool , loss_rate : &AtomicU8) -> tokio::io::Result<usize>{
+        if enable_loss.load(Ordering::Relaxed){
+            let mut rng = rand::thread_rng();
+            let i: u8 = rng.gen_range(1..11);
+            if i > loss_rate.load(Ordering::Relaxed) {
+                raknet_log!("loss packet");
+                return Ok(0);
+            }
+        }
+        s.send_to(buf, target).await
+    }
+
     pub async fn connect(addr : &SocketAddr) -> Result<Self>{
 
         let guid : u64 = rand::random();
@@ -177,7 +203,7 @@ impl RaknetSocket {
         let buf = write_packet_connection_request(&packet).await.unwrap();
 
         let mut sendq1 = sendq.lock().await;
-        sendq1.insert(Reliability::ReliableOrdered, &buf, cur_timestamp_millis());
+        sendq1.insert(Reliability::ReliableOrdered, &buf)?;
         std::mem::drop(sendq1);
 
         let (user_data_sender , user_data_receiver) =  channel::<Vec<u8>>(100);
@@ -226,6 +252,9 @@ impl RaknetSocket {
             recvq : Arc::new(Mutex::new(RecvQ::new())),
             sendq,
             connected,
+            last_heartbeat_time : Arc::new(AtomicI64::new(cur_timestamp_millis())),
+            enable_loss : Arc::new(AtomicBool::new(false)),
+            loss_rate : Arc::new(AtomicU8::new(0))
         };
 
         ret.start_receiver(receiver , user_data_sender);
@@ -240,6 +269,9 @@ impl RaknetSocket {
         let sendq = self.sendq.clone();
         let socket = self.s.clone();
         let recvq = self.recvq.clone();
+        let enable_loss = self.enable_loss.clone();
+        let loss_rate = self.loss_rate.clone();
+        let last_heartbeat_time = self.last_heartbeat_time.clone();
         tokio::spawn(async move {
             loop{
                 if !connected.load(Ordering::Relaxed){
@@ -254,6 +286,8 @@ impl RaknetSocket {
                     },
                 };
 
+                last_heartbeat_time.store(cur_timestamp_millis(), Ordering::Relaxed);
+
                 if PacketID::from(buf[0]).unwrap() == PacketID::Disconnect{
                     connected.store(false, Ordering::Relaxed);
                     break;
@@ -300,7 +334,7 @@ impl RaknetSocket {
                     for frame in frames.frames{
                         recvq.insert(frame).unwrap();
 
-                        for f in recvq.flush(){
+                        for f in recvq.flush(&peer_addr){
                             if !RaknetSocket::handle(&f , &peer_addr ,&local_addr, &sendq, &user_data_sender).await.unwrap(){
                                 connected.store(false, Ordering::Relaxed);
                                 is_break = true;
@@ -322,7 +356,7 @@ impl RaknetSocket {
                         };
 
                         let buf = write_packet_nack(&nack).await.unwrap();
-                        match socket.send_to(&buf, peer_addr).await{
+                        match RaknetSocket::sendto(&socket , &buf, &peer_addr , &enable_loss , &loss_rate).await{
                             Ok(_) => {},
                             Err(_) => {
                                 break;
@@ -345,6 +379,9 @@ impl RaknetSocket {
         let sendq = self.sendq.clone();
         let recvq = self.recvq.clone();
         let mut last_monitor_tick = cur_timestamp_millis();
+        let enable_loss = self.enable_loss.clone();
+        let loss_rate = self.loss_rate.clone();
+        let last_heartbeat_time = self.last_heartbeat_time.clone();
         tokio::spawn(async move {
             loop{
                 sleep(std::time::Duration::from_millis(50)).await;
@@ -365,29 +402,36 @@ impl RaknetSocket {
                     };
 
                     let buf = write_packet_ack(&packet).await.unwrap();
-                    s.send_to(&buf, peer_addr).await.unwrap();
+                    RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss  , &loss_rate).await.unwrap();
                 }
                 
                 //flush sendq
                 let mut sendq = sendq.lock().await;
-                sendq.tick(cur_timestamp_millis());
-                for f in sendq.flush(){
+                for f in sendq.flush(cur_timestamp_millis(), &peer_addr){
                     let data = f.serialize().await.unwrap();
-                    s.send_to(&data, peer_addr).await.unwrap();
+                    RaknetSocket::sendto(&s , &data, &peer_addr , &enable_loss  , &loss_rate).await.unwrap();
                 }
 
                 //monitor log
                 if cur_timestamp_millis() - last_monitor_tick > 10000{
-                    raknet_log!("peer addr : {} ,repeat queue size : {} , reliable packet queue size : {} , recvq size : {} , recvq fragment size : {}" , 
+                    raknet_log!("peer addr : {} , sendq size : {} , sentq size : {} , recvq size : {} ,  recvq fragment size : {} , ordered queue size : {} - {:?}" , 
                         peer_addr,
-                        sendq.get_repeat_queue_size() , 
                         sendq.get_reliable_queue_size(),
+                        sendq.get_sent_queue_size(),
                         recvq.get_size(),
-                        recvq.get_fragment_queue_size()
+                        recvq.get_fragment_queue_size(),
+                        recvq.get_ordered_packet(),
+                        recvq.get_ordered_keys()
                     );
                     last_monitor_tick = cur_timestamp_millis();
                 }
 
+                // if exceed 60s not received any packet will close connection.
+                if cur_timestamp_millis() - last_heartbeat_time.load(Ordering::Relaxed) > RECEIVE_TIMEOUT{
+                    connected.store(false, Ordering::Relaxed);
+                    break;
+                }
+
                 if !connected.load(Ordering::Relaxed){
                     break;
                 }
@@ -416,7 +460,7 @@ impl RaknetSocket {
             Err(_) => return Err(RaknetError::BindAdreesError),
         };
 
-        let buf = write_packet_ping(&packet).await.unwrap();
+        let buf = write_packet_ping(&packet).await?;
 
         s.send_to(buf.as_slice(), addr).await.unwrap();
 
@@ -435,7 +479,7 @@ impl RaknetSocket {
     }
 
     pub async fn close(&mut self) -> Result<()>{
-        self.sendq.lock().await.insert(Reliability::Reliable, &[PacketID::Disconnect.to_u8()] , cur_timestamp_millis());
+        self.sendq.lock().await.insert(Reliability::Reliable, &[PacketID::Disconnect.to_u8()])?;
         self.connected.store(false, Ordering::Relaxed);
         Ok(())
     }
@@ -446,7 +490,14 @@ impl RaknetSocket {
             return Err(RaknetError::ConnectionClosed);
         }
 
-        self.sendq.lock().await.insert(r , buf, cur_timestamp_millis());
+        self.sendq.lock().await.insert(r , buf)?;
+
+        //flush sendq
+        let mut sendq = self.sendq.lock().await;
+        for f in sendq.flush(cur_timestamp_millis(), &self.peer_addr){
+            let data = f.serialize().await.unwrap();
+            RaknetSocket::sendto(&self.s , &data, &self.peer_addr , &self.enable_loss , &self.loss_rate).await.unwrap();
+        }
         Ok(())
     }
 
@@ -457,12 +508,12 @@ impl RaknetSocket {
         }
 
         match self.user_data_receiver.recv().await{
-            Some(p) => return Ok(p),
+            Some(p) => Ok(p),
             None => {
                 if !self.connected.load(Ordering::Relaxed){
                     return Err(RaknetError::ConnectionClosed);
                 }
-                return Err(RaknetError::RecvFromError);
+                Err(RaknetError::RecvFromError)
             },
         }
 
